import sqlite3
import xml.etree.ElementTree as ET
from datetime import datetime
import os

#--- CONFIGURATION ---
DBName = "/config/www/epg/epg_data.db"
XMLFile = "/config/www/epg/epg.xml"
HTMLFilelocation = "/config/www/epg/"
LOGOFolder = "logos/"
HA_BaseURL = "https://192.168.123.88:8123/"
HA_EpgURL = "https://192.168.123.88:8123/api/states/input_text.epg_recid"

conn = sqlite3.connect(DBName)
#--- CLEAR EPG TABLE and old html files ---
conn.execute ("DELETE FROM epg_data")
conn.commit()

for filename in os.listdir(HTMLFilelocation):
  if filename.endswith(".html"):
    os.remove(HTMLFilelocation + filename)


#--- GET EPG FROM XML AND STORE IT IN SQLITE3 DB ---
#--- DB: channel / start / stop / title / subtitle / description / episode / ID (Auto)
f = open(XMLFile,"r")
recid = 0
for x in f:
  recid = recid + 1
  #--- only process TV guide, skip channel list in file ---
  if "<programme" in x:
    tree = ET.ElementTree(ET.fromstring(x.strip()))
    root= ET.fromstring(x.strip())

    dta_title = root.findtext("title")
    dta_subtitle = root.findtext("Sub-title")
    dta_description =root.findtext("desc")
    dta_episode = root.findtext("episode-num")

    for element_desc,element_value in root.items():
      if element_desc == "start":
        dta_start = element_value
      if element_desc == "stop":
        dta_stop = element_value
      if element_desc == "channel":
        dta_channel = element_value

    sqldata = [dta_channel,dta_start,dta_stop,dta_title,dta_subtitle,dta_description,dta_episode,str(recid)]
    sql = "INSERT INTO epg_data (channel,start,stop,title,subtitle,description,episode,recid) VALUES(?,?,?,?,?,?,?,?)"
    conn.execute(sql,sqldata)
conn.commit()
f.close()

#--- get list of dates in DB ---
datelist = conn.execute("SELECT DISTINCT substr(start,0,9) FROM epg_data")
for dta in datelist:
  #--- BEGIN: HEADER SECTION ---
  f = open(HTMLFilelocation + dta[0] + ".html","w")
  SHOWDate = dta[0]
  StrDate = SHOWDate[6:8] +"/" + SHOWDate[4:6] + "/" + SHOWDate[0:4]
  dt = datetime.strptime(StrDate,"%d/%m/%Y")
  NumDay = int(dt.strftime("%w"))
  DayName = "xxx"
  if NumDay == 1:
    DayName = "Maandag"
  if NumDay == 2:
    DayName = "Dinsdag"
  if NumDay == 3:
    DayName = "Woensdag"
  if NumDay == 4:
    DayName = "Donderdag"
  if NumDay == 5:
    DayName = "Vrijdag"
  if NumDay == 6:
    DayName = "Zaterdag"
  if NumDay == 0:
    DayName = "Zondag"
  StrDate = DayName + " - " + StrDate
  f.write("<HTML>                                                                                       \n" \
        + "  <script src='https://ajax.googleapis.com/ajax/libs/jquery/2.1.3/jquery.min.js'></script>   \n" \
        + "  <script src='/js/my.js'></script>                                                          \n" \
        + "  <SCRIPT language='javascript' type='text/javascript'></script>                             \n" \
        + "  <SCRIPT>                                                                                   \n" \
        + "    $(document).ready(function ()                                                            \n" \
        + "    {                                                                                        \n" \
        + "      $('#leftx').scroll(function ()                                                         \n" \
        + "       {                                                                                     \n" \
        + "         $('#right').scrollTop($('#leftx').scrollTop());                                     \n" \
        + "       });                                                                                   \n" \
        + "       $('#right').scroll(function ()                                                        \n" \
        + "        {                                                                                    \n" \
        + "	  $('#leftx').scrollTop($('#right').scrollTop());                                           \n" \
        + "	  $('#righttop').scrollLeft($('#right').scrollLeft());                                      \n" \
        + "        });                                                                                  \n" \
        + "       $('#righttop').scroll(function ()                                                     \n" \
        + "        {                                                                                    \n" \
        + "	  $('#right').scrollLeft($('#righttop').scrollLeft());                                      \n" \
        + "       });                                                                                   \n" \
        + "  });                                                                                        \n" \
        + " </SCRIPT>                                                                                   \n" \
        + "<HEAD>                                                                                       \n" \
        + "<STYLE>                                                                                      \n" \
        + "  #toptopxx {                                                                                \n" \
        + "             width: 1335px;                                                                  \n" \
        + "             height: 50px;                                                                   \n" \
        + "             float: left;                                                                    \n" \
        + "             clear: both;                                                                    \n" \
        + "             }                                                                               \n" \
        + "  #lefttopx {                                                                                \n" \
        + "             width: 24%;                                                                     \n" \
        + "             height: 50px;                                                                   \n" \
        + "             float: left;                                                                    \n" \
        + "             text-align: left;                                                               \n" \
        + "             overflow-y: hidden;                                                             \n" \
        + "             overflow-x: hidden;                                                             \n" \
        + "             left: 0;                                                                        \n" \
        + "             }                                                                               \n" \
        + "  #righttop {                                                                                \n" \
        + "             width: 76%;                                                                     \n" \
        + "             height: 50px;                                                                   \n" \
        + "             float: right;                                                                   \n" \
        + "             text-align: left;                                                               \n" \
        + "             overflow-y: scroll;                                                             \n" \
        + "             overflow-x: hidden;                                                             \n" \
        + "             right: 0;                                                                       \n" \
        + "            }                                                                                \n" \
        + "  #topxx {                                                                                   \n" \
        + "          width: 1335px;                                                                     \n" \
        + "          height: 680px;                                                                     \n" \
        + "          float: left;                                                                       \n" \
        + "          clear: both;                                                                       \n" \
        + "         }                                                                                   \n" \
        + "  #leftx {                                                                                   \n" \
        + "          width: 24%;                                                                        \n" \
        + "          height: 664px;                                                                     \n" \
        + "          float: left;                                                                       \n" \
        + "          text-align: left;                                                                  \n" \
        + "          overflow-y: hidden;                                                                \n" \
        + "          overflow-x: hidden;                                                                \n" \
        + "          left: 0;                                                                           \n" \
        + "         }                                                                                   \n" \
        + "  #right {                                                                                   \n" \
        + "          width: 76%;                                                                        \n" \
        + "          height: 680px;                                                                     \n" \
        + "          float: right;                                                                      \n" \
        + "          text-align: left;                                                                  \n" \
        + "          overflow-y: auto;                                                                  \n" \
        + "          right: 0;                                                                          \n" \
        + "         }                                                                                   \n" \
        + "  table {                                                                                    \n" \
        + "         background-color: transparent; border-spacing: 1px;                                 \n" \
        + "        }                                                                                    \n" \
        + "  th, td {                                                                                   \n" \
        + "          padding: 5px; text-align:left; color: white; border: 0px;                          \n" \
        + "         }                                                                                   \n" \
        + "  body {                                                                                     \n" \
        + "        background-image: url('iptv.jpg');                                                   \n" \
        + "       }                                                                                     \n" \
        + "  tr {                                                                                       \n" \
        + "      background-color:rgba(64,64,64,80%); color: white; padding: 5px;                       \n" \
        + "     }                                                                                       \n" \
        + "  #even {                                                                                    \n" \
        + "          background-color:rgba(96,96,96,80%); color: white; padding: 5px;                   \n" \
        + "        }                                                                                    \n" \
        + "  #titel {                                                                                   \n" \
        + "          height:50px; color: white; padding: 5px; background-color: rgba(96,0,0,80%);       \n" \
        + "         }                                                                                   \n" \
        + "  a:link    {color: white; text-decoration: none;}                                           \n" \
        + "  a:visited {color: white; text-decoration: none;}                                           \n" \
        + "  a:hover   {color: white; text-decoration: none;}                                           \n" \
        + "  a:active  {color: white; text-decoration: none;}                                           \n" \
        + "  .clickable-text {cursor: pointer; margin-bottom: 10px; }                                   \n "\
        + "  #snackbar {                                                                                                                        \n" \
        + "             visibility: hidden; min-width: 300px; margin-left: -150px; background-color: #000000;                                   \n" \
        + "             color: #FFFFFF; text-align: center; border-radius: 15px; padding: 20px; position: fixed;                                \n" \
        + "             z-index: 1; left: 50%; top: 30px;                                                                                       \n" \
        + "            }                                                                                                                        \n" \
        + "  #snackbar.show {                                                                                                                   \n" \
        + "                  visibility: visible; -webkit-animation: fadein 0.5s, fadeout 0.5s 2.5s; animation: fadein 0.5s, fadeout 0.5s 2.5s; \n" \
        + "                 }                                                                                                                   \n" \
        + "  @-webkit-keyframes fadein {                                                                                                        \n" \
        + " from {top: 30px; opacity: 0;} to {top: 30px; opacity: 1;}                                                                           \n" \
        + "                            }                                                                                                        \n" \
        + "  @keyframes fadein {                                                                                                                \n" \
        + "                     from {top: 30px; opacity: 0;} to {top: 30px; opacity: 1;}                                                       \n" \
        + "                    }                                                                                                                \n" \
        + "  @-webkit-keyframes fadeout {                                                                                                       \n" \
        + "                              from {top: 30px; opacity: 1;} to {top: 30px; opacity: 0;}                                              \n" \
        + "                             }                                                                                                       \n" \
        + "  @keyframes fadeout {                                                                                                               \n" \
        + "                      from {top: 30px; opacity: 1;} to {top: 30px; opacity: 0;}                                                      \n" \
        + "                     }                                                                                                               \n" \
        + "</STYLE>                                                                                     \n" \
        + "</HEAD>                                                                                      \n" \
        + "<div id='toptopxx'>                                                                          \n" \
        + "  <div id='lefttopx'>                                                                        \n" \
        + "     <table style='width:330px;'><tr id='titel'><th style='text-align:center;background-color:rgba(0,0,64,80%);'>" \
        + StrDate + "</th></tr></table>                                                                 \n" \
#        + "margin-left:auto;display:block;width:100px;'></td></tr></table>                              \n" \
        + "   </div>                                                                                    \n" \
        + "   <div id='righttop'>                                                                       \n" \
        + "      <table style='width:36000px;'><tr id='titel' > " )

  #--- BUILD SCROLLABLE TIMELINE HEADER WITH ANCHORS ---
  for x in range(24):
    l = "0" + str(x)
    l = l[-2:]
    f.write("<th style='width750px;background-color:rgba(64,0,0,80%);'><div id='" + l + "00'>" + l + ":00</div></th>" \
          + "<th style='width750px;background-color:rgba(96,0,0,80%);'><div id='" + l + "30'>" + l + ":30</div></th>")
  f.write(" </tr></table>                                                                               \n" \
        + "	</div>                                                                                      \n" \
        + "</div>                                                                                       \n" \
        + "<div id='topxx'>                                                                             \n" )

  #--- LEFT SIDE : CHANNELS AND LOGOS ---
  f.write("   <div id='leftx'>                                                                          \n")
  cursor = conn.execute("SELECT channelname, logoname from epg_channel WHERE channelname IN (SELECT DISTINCT channel FROM epg_data) ORDER BY sortorder ASC")
  count = 0
  for row in cursor:
    channel=row[0]
    logo=row[1]
    channel=channel.strip()
    logo=logo.strip()
    count = count + 1
    f.write("     <table style='width:330px;'>")
    posx = channel.rfind(".")
    dispchannel = channel[:posx]
    if count % 2:
      f.write("<tr id='even'><td style='width:140px;height:94px;'><img src='" + LOGOFolder + logo + "' height=20px></td><td>" +dispchannel + "</td></tr></table> \n")
    else:
      f.write("<tr><td style='width:140px;height:94px;'><img src='" + LOGOFolder + logo + "' height=20px></td><td>" + dispchannel + "</td></tr></table> \n")
  f.write("   </div> \n")

  #--- RIGHT SIDE: TIMELINE ---
  f.write("   <div id='right'>                                                                          \n")
  cursor = conn.execute("SELECT DTA.channel,DTA.start,DTA.stop,DTA.title,DTA.subtitle,DTA.description,DTA.episode,DTA.recid,CH.sortorder FROM epg_data as DTA, epg_channel as CH WHERE CH.channelname = DTA.channel ORDER BY CH.sortorder ASC, DTA.start ASC")
  oldchannel = ""
  counter = 0
  for row in cursor:
    dta_channel = row[0]
    dta_start = row[1]
    dta_stop = row[2]
    dta_title = row[3]
    dta_subtitle = row[4]
    dta_description = row[5]
    dta_episode = row[6]
    dta_recid = row[7]
    #--- ONLY DO SOMETHING FOR GIVEN DATE ---
    if dta_start[:8] ==  SHOWDate[:8]:
      #--- do logic for new row ---
      newline = False
      if oldchannel != dta_channel:
        counter =  counter + 1
        newline = True
        if counter % 2:
          StrRowID = " id='even' "
        else:
          StrRowID = ""
        #--- close previous row if oldchannel is empty ---
        if oldchannel != "":
          f.write(" </tr></table> \n" \
                + "      <table style='width:36000px;'><tr" + StrRowID +  "> ")
        else:
          f.write( "      <table style='width:36000px;'><tr" + StrRowID + "> ")
      oldchannel = dta_channel
      #--- format start- and stoptime to show in timeline ---
      starttime = dta_start[8:10] + ":" + dta_start[10:12]
      stoptime = dta_stop[8:10] + ":" + dta_stop[10:12]
      startdate = dta_start[:8]
      stopdate = dta_stop[:8]
      hourstop = int(dta_stop[8:10])
      minutestop = int(dta_stop[10:12])
      hourstart = int(dta_start[8:10])
      minutestart = int(dta_start[10:12])
      if  startdate == stopdate:
        difference = datetime.strptime(str(hourstop) + ":" + str(minutestop),"%H:%M") - datetime.strptime(str(hourstart) + ":" + str(minutestart),"%H:%M") 
        totalminutes = difference.total_seconds() / 60
        houradd = 0
        minuteadd = 0
      else:
        difference = datetime.strptime(str(hourstart) + ":" + str(minutestart), "%H:%M") - datetime.strptime( "00:00" , "%H:%M")
        totalminutes = 1440 - (difference.total_seconds() / 60)
        houradd = hourstop
        minuteadd =minutestop
      if newline:
        drawpixels  = ((hourstart *60) + minutestart) * 25
        if starttime != "00:00":
          f.write("<td style='width:" + str(int(drawpixels)) + "px;height:94px;background-color:rgba(128,0,0,25%);' ></td>")
      drawpixels = totalminutes * 25
      totalminutes = totalminutes + minuteadd + (houradd * 60)
      totalhours = int(totalminutes // 60)
      totalminutes = totalminutes - (totalhours * 60)
      totalminutes = int(totalminutes)
      strtime = str(totalhours) + "u " + str(totalminutes) + "m"
      if len(dta_description) > 150:
        dots = " ..."
      else:
        dots = ""
      #--- dont show details when block is not wide enough ---
      if drawpixels < 200:
        dta_description = ""
        dots = ""
      f.write("<td style='width:" + str(int(drawpixels)) + "px;height:94px;vertical-align: top;'><B><div class='clickable-text' onclick="+ chr(34) + "sendRequest('" + str(dta_recid) + "')" + chr(34) + ">" + dta_title + "</div></B>" )
      if totalhours < 1 and totalminutes < 30:
        f.write("...")      
      else:
        f.write("<font color='#AAAAFF'>" + starttime + " - " + stoptime +  "</font> " )
        f.write("<font color='#FFAAAA'>(" + strtime + ")</font><BR CLEAR=ALL>")
        f.write("<font size='-1'><font color='#FFFF88'>" + dta_description[:150] + dots + "</font></font></td>")
  f.write("</tr></table> \n" \
        + "    </div></div>")

  #--- END: FOOTER ---
  f.write("<div id='topxx'>                                                      \n" \
        + "  <div id='right'>                                                    \n" \
        + "    <table><tr id='footer'>                                           \n" )

  lst = conn.execute("SELECT DISTINCT substr(start,0,9) FROM epg_data")
  for datum in lst:
    dat = datum[0]
    if dat == SHOWDate:
      clr = "#4444FF"
    else:
      clr = "#000044"
    datx = dat[6:8] +"/" + dat[4:6] + "/" + dat[0:4]
    f.write("<th style='background-color:" + clr + ";color:#FFFFFF;width:250px;text-align:center;'><a href='" + datum[0] + ".html'>" + datx  + "</a></th>")
  f.write("    </tr></table>                                                                            \n" \
        + "  </div>                                                                                     \n" \
        + "</div>                                                                                       \n" \
        + "      <script>                                                                               \n" \
        + "        function sendRequest(state) {                                                        \n" \
        + "           fetch('" + HA_EpgURL + "', {            \n" \
        + "             method: 'POST',                                                                 \n" \
        + "               headers: {                                                                    \n" \
        + "                 'Authorization': 'Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJiNzViYWViN2MzMjI0YTlmYmRlOWYxZDQxMWIzN2Y5NSIsImlhdCI6MTcyMjIzNzk1OSwiZXhwIjoyMDM3NTk3OTU5fQ.AYMl7ztRJv6vUaM06ZtwF-7C94DQ_vP08zOVfUrTVrk',\n" \
        + "                 'Content-Type': 'application/json'                                          \n" \
        + "               },                                                                            \n" \
        + "               body: JSON.stringify({ state: state }),                                       \n" \
        + "           })                                                                                \n" \
        + "           .then(response => {                                                               \n" \
        + "              if (!response.ok) {                                                            \n" \
        + "                throw new Error('Network response was not ok');                              \n" \
        + "              }                                                                              \n" \
        + "              return response.json();                                                        \n" \
        + "           })                                                                                \n" \
        + "           .then(data => {                                                                   \n" \
        + "              console.log('Success:', data);                                                 \n" \
        + "           })                                                                                \n" \
        + "           .catch(error => {                                                                 \n" \
        + "              console.error('Error:', error);                                                \n" \
        + "           });                                                                               \n" \
        + "           var x = document.getElementById('snackbar');                                      \n" \
        + "           x.className = 'show';                                                             \n" \
        + "           setTimeout(function(){ x.className = x.className.replace('show', ''); }, 3000);   \n" \
        + "        }                                                                                    \n" \
        + "      </script>                                                                              \n" \
        + "</BODY>                                                                                      \n" \
        + "</HTML>                                                                                      \n")
  f.close()

#--- @@@ THE END - CLEANUP @@@ ---
conn.close()

